import QtQuick 2.5
import com.zuragon.ViCANdo 1.0
import QtQuick.Extras 1.4

Rectangle {
        width: 800
        height: 600
        color: "black"
        id: mobileeye_viewer
        visible: true

        property bool fcw_on: false
        property bool error_active: false
        property int tsr_warning_level: 0
        property bool brakes: false
        property bool left_blink: false
        property bool right_blink: false
        property real speed: 0
        property bool left_ldw_on: false
        property bool right_ldw_on: false
        property bool low_beam: false
        property bool high_beam: false
        property bool high_beam_availabled: false
        property bool low_beam_availabled: false

        property int sign_type_1: 0
        property int sign_type_2: 0
        property int sign_type_3: 0
        property int sign_type_4: 0
        property int supplementary_sign_type_1: 0
        property int supplementary_sign_type_2: 0
        property int supplementary_sign_type_3: 0
        property int supplementary_sign_type_4: 0


        Text{
            id: title
            text: " ViCANdo QML example for MobileEye sensor "
            font.family: "Helvetica"
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
            font.pointSize: 18
            color: "white"
        }

        Text {
            id: left_blink_text
            text: "Left blink"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: title.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        StatusIndicator {
            id: left_blink_indicator
            y: 140
            color: "red"
            active: left_blink
            anchors.top: left_blink_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: fcw_on_text
            text: "Forward collision warning"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: title.bottom
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
        }
        StatusIndicator {
            id: fcw_on_indicator
            y: 140
            color: "red"
            active: fcw_on
            anchors.top: fcw_on_text.bottom
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
        }
        Text {
            id: right_blink_text
            anchors.top: title.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            text: "Right blink"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
        }
        StatusIndicator {
            id: right_blink_indicator
            y: 140
            color: "red"
            active: right_blink
            anchors.top:right_blink_text.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
        }


        Text {
            id: left_ldw_on_text
            text: "Left LDW"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: left_blink_indicator.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        StatusIndicator {
            id: left_ldw_on_indicator
            y: 140
            color: "red"
            active: left_ldw_on
            anchors.top: left_ldw_on_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text{
            id: speed_text
            text: "Speed " + speed + " km/h"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: fcw_on_indicator.bottom
            anchors.topMargin: 10
            anchors.horizontalCenter: parent.horizontalCenter
        }
        CircularGauge {
            id: speed_gauge
            minimumValue: 0
            maximumValue: 255
            stepSize: 1
            value: speed
            anchors.top: speed_text.bottom
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
        }

        Text {
            id: right_ldw_on_text
            anchors.top: right_blink_indicator.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
            text: "Right LDW"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
        }
        StatusIndicator {
            id: right_ldw_on_indicator
            y: 140
            color: "red"
            active: right_ldw_on
            anchors.top:right_ldw_on_text.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 5
        }

        Text {
            id: high_beam_text
            text: "High Beam"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: left_ldw_on_indicator.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        StatusIndicator {
            id: high_beam_indicator
            y: 140
            color: "blue"
            active: high_beam_availabled && high_beam
            anchors.top:high_beam_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }

        Text {
            id: low_beam_text
            text: "Low Beam"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: high_beam_indicator.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        StatusIndicator {
            id: low_beam_indicator
            y: 140
            color: "green"
            active: low_beam_availabled && low_beam
            anchors.top:low_beam_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }

        Text {
            id: brake_text
            text: "Brakes"
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: low_beam_indicator.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        StatusIndicator {
            id: brakes_indicator
            y: 140
            color: "red"
            active: brakes
            anchors.top:brake_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }


        SourceEventListener {
                        id: listen_to_can_msg
                        source: project.findObject("can1")
                        dbc: project.findObject("mobileeye_ftp_all_protocol")
                        filter: ["AWS","signal_car", "TSR_detail_1", "TSR_detail_2", "TSR_detail_3", "TSR_detail_4", "TSR_detail_5", "TSR_detail_6", "TSR_detail_7", "Vision_only_TSR" ]
                        onFrameDataReceived: {

                            //Handling the received signals
                             if( name == "AWS"){
                                fcw_on = data["FCW_on"]
                                error_active = data["Error_Active"]
                                tsr_warning_level = data["TSR_warning_level"]
                                left_ldw_on = data["LLDW_on"]
                                right_ldw_on = data["RLDW_on"]

                             }
                             if( name == "signal_car"){
                                 brakes = data["Brakes"]
                                 left_blink = data["Left_blink"]
                                 right_blink = data["right_blink"]
                                 speed = data["speed"]
                                 high_beam = data["HighBeam"]
                                 low_beam = data["LowBeam"]
                                 high_beam_availabled = data["HighBeam_availabled"]
                                 low_beam_availabled = data["LowBeam_availabled"]

                             }
                             if(name == "Vision_only_TSR"){
                                 sign_type_1 = data["Sign_type_1"]
                                 sign_type_2 = data["Sign_type_2"]
                                 sign_type_3 = data["Sign_type_3"]
                                 sign_type_4 = data["Sign_type_4"]
                                 supplementary_sign_type_1 = data["Supplementary_sign_type_1"]
                                 supplementary_sign_type_2 = data["Supplementary_sign_type_2"]
                                 supplementary_sign_type_3 = data["Supplementary_sign_type_3"]
                                 supplementary_sign_type_4 = data["Supplementary_sign_type_4"]
                             }

                        }
        }


        Text {
            id: tsr_warning_level_text
            text: "TSR warning level:" + tsr_warning_level + ". " + getTsrWarningString(tsr_warning_level)
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: brakes_indicator.bottom
            anchors.topMargin: 50
            anchors.left: parent.left
            anchors.leftMargin: 5
        }


        function getTsrWarningString(warninglevel){
            switch(warninglevel){
            case 0: return "Speed within limit"
            case 1: return "Warning: Speeding 0-5 km/h"
            case 2: return "Warning: Speeding 5-10 km/h"
            case 3: return "Warning: Speeding 10-15 km/h"
            case 4: return "Warning: Speeding 15-20 km/h"
            case 5: return "Warning: Speeding 20-25 km/h"
            case 6: return "Warning: Speeding 25-30 km/h"
            case 7: return "Warning: Speeding 30+ km/h"
            default: return "Invalid TSR warning level"
            }
        }


        Text {
            id: tsr1_text
            text: "TSR 1 sign type: " + getSignTypeString (sign_type_1)
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: tsr_warning_level_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: tsr2_text
            text: "TSR 2 sign type :" + getSignTypeString (sign_type_2)
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: tsr1_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: tsr3_text
            text: "TSR 3 sign type :" + getSignTypeString (sign_type_3)
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: tsr2_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: tsr4_text
            text: "TSR 4 sign type :" + getSignTypeString (sign_type_4)
            font.family: "Helvetica"
            font.pointSize: 12
            color: "white"
            anchors.top: tsr3_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }



        // Ref: Mobileye_C2_Standard_CAN_Output_Protocol_with_TSR.doc, verson 1.1
        function getSignTypeString(sign_type){
            switch(sign_type){
                  case 0  : return "standard regular 10";
                  case 1  : return "standard regular 20";
                  case 2  : return "standard regular 30";
                  case 3  : return "standard regular 40";
                  case 4  : return "standard regular 50";
                  case 5  : return "standard regular 60";
                  case 6  : return "standard regular 70";
                  case 7  : return "standard regular 80";
                  case 8  : return "standard regular 90";
                  case 9  : return "standard regular 100";
                  case 10  : return "standard regular 110";
                  case 11  : return "standard regular 120";
                  case 12  : return "standard regular 130";
                  case 13  : return "standard regular 140";
                  case 20  : return "standard regular end restriction of number e.g 60 end of restriction";
                  case 28  : return "standard electronic 10";
                  case 29  : return "standard electronic 20";
                  case 30  : return "standard electronic 30";
                  case 31  : return "standard electronic 40";
                  case 32  : return "standard electronic 50";
                  case 33  : return "standard electronic 60";
                  case 34  : return "standard electronic 70";
                  case 35  : return "standard electronic 80";
                  case 36  : return "standard electronic 90";
                  case 37  : return "standard electronic 100";
                  case 38  : return "standard electronic 110";
                  case 39  : return "standard electronic 120";
                  case 40  : return "standard electronic 130";
                  case 41  : return "standard electronic 140";
                  case 50  : return "standard electronic end restriction of number e.g 60 end of restriction";
                  case 64  : return "standard regular general end all restriction";
                  case 65  : return "standard electronic general end all restriction";
                  case 100  : return "standard regular 5";
                  case 101  : return "standard regular 15";
                  case 102  : return "standard regular 25";
                  case 103  : return "standard regular 35";
                  case 104  : return "standard regular 45";
                  case 105  : return "standard regular 55";
                  case 106  : return "standard regular 65";
                  case 107  : return "standard regular 75";
                  case 108  : return "standard regular 85";
                  case 109  : return "standard regular 95";
                  case 110  : return "standard regular 105";
                  case 111  : return "standard regular 115";
                  case 112  : return "standard regular 125";
                  case 113  : return "standard regular 135";
                  case 114  : return "standard regular 145";
                  case 115  : return "standard electronic 5";
                  case 116  : return "standard electronic 15";
                  case 117  : return "standard electronic 25";
                  case 118  : return "standard electronic 35";
                  case 119  : return "standard electronic 45";
                  case 120  : return "standard electronic 55";
                  case 121  : return "standard electronic 65";
                  case 122  : return "standard electronic 75";
                  case 123  : return "standard electronic 85";
                  case 124  : return "standard electronic 95";
                  case 125  : return "standard electronic 105";
                  case 126  : return "standard electronic 115";
                  case 127  : return "standard electronic 125";
                  case 128  : return "standard electronic 135";
                  case 129  : return "standard electronic 145";
                  case 171  : return "standard regular motorWay begin";
                  case 172  : return "standard regular end of fMotorWay";
                  case 173  : return "standard regular expressWay begin";
                  case 174  : return "standard regular end of ExpressWay";
                  case 175  : return "standard regular Playground area begin";
                  case 176  : return "standard regular End of playground area";
                  case 200  : return "standard regular no passing start";
                  case 201  : return "standard regular end of no passing";
                  case 220  : return "standard electronic no passing start";
                  case 221  : return "standard electronic end of no passing";
                  case 254 : return " No sign detected";
                  case 255 : return " e_invalid_sign";
                  default:
                      return "Invalid TSR sign type";
            }


        }


}
